@@ -37,7 +37,7 @@
 //////////////////////////////////////////////////////////////////////
 // static variable
 //////////////////////////////////////////////////////////////////////
-static vector_obj* files = NULL;
+static vector_obj* files;
 
 //////////////////////////////////////////////////////////////////////
 // readline
@@ -263,6 +263,14 @@ void migemo_init()
 
 void migemo_final()
 {
+    hash_it* it = hash_loop_begin(gMigemoRegsCache);
+    while(it) {
+       regex_t* obj = hash_loop_item(it);
+       onig_free(obj);
+ 
+       it = hash_loop_next(it);
+    }
+    hash_delete(gMigemoRegsCache);
     onig_end();
     migemo_close(gMigemo);
 }
@@ -332,7 +340,7 @@ char* readline_file_completion_migemo_match(char* text, char* file, regex_t* reg
         *_p2 = 0;
 
         regex_t* reg;
-        if(strlen(p2) == 2) {
+        if(strlen(p2) <= 2) {
             reg = hash_item(gMigemoRegsCache, p2);
 
             if(reg == NULL) {
@@ -364,6 +372,9 @@ char* readline_file_completion_migemo_match(char* text, char* file, regex_t* reg
         migemo_release(gMigemo, (unsigned char*) p);
 
         if(r != ONIG_NORMAL) {
+            if(strlen(p2) > 2) { onig_free(reg); }
+
+            FREE(p2);
             return NULL;
         }
         OnigRegion* region = onig_region_new();
@@ -374,7 +385,9 @@ char* readline_file_completion_migemo_match(char* text, char* file, regex_t* reg
 
         onig_region_free(region, 1);
 
-        if(strlen(p2) > 1) { onig_free(reg); }
+        if(strlen(p2) > 2) { onig_free(reg); }
+
+        FREE(p2);
     }
 
     if(r2 == 0) {
@@ -413,12 +426,13 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         index = 0;
 
         /// migemoのクェリーと正規表現コンパイル結果を得る
-        if(reg) { if(hash_key(gMigemoRegsCache, reg) == NULL) { onig_free(reg);} }
+        if(reg) { if(hash_key(gMigemoRegsCache, reg) == NULL) { onig_free(reg); reg = NULL; } }
 
         OnigUChar* p = migemo_query(gMigemo, text2);
 
         if(p == NULL) {
-           return NULL;
+            FREE(text2);
+            return NULL;
         }
 
         int r;
@@ -440,7 +454,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         *_p2 = 0;
 
 
-        if(strlen(p2) == 2) {
+        if(strlen(p2) <= 2) {
             reg = hash_item(gMigemoRegsCache, p2);
 
             if(reg == NULL) {
@@ -469,9 +483,12 @@ char* readline_file_completion_migemo(const char* text, int rstat)
             }
         }
 
+        FREE(p2);
+
         migemo_release(gMigemo, (unsigned char*) p);
 
         if(r != ONIG_NORMAL) {
+            onig_free(reg);
             reg = NULL;
         }
 
@@ -482,6 +499,8 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         parentname2(tmp, string_length(gEditingDir) + 1, string_c_str(gEditingDir));
         string_put(gEditingDir, tmp);
 
+        FREE(tmp);
+
         /// チルダから始まっている場合 ///
         if(string_c_str(gEditingDir)[0] == '~') {
             char* p = string_c_str(gEditingDir);
@@ -527,12 +546,12 @@ char* readline_file_completion_migemo(const char* text, int rstat)
         }
 
         /// 補完候補を初期化 ///
-        if(files == NULL) {
-            files = VECTOR_NEW(50);
-        }
-        else {
-            vector_clear(files);
+        int i;
+        for(i=0;i<vector_size(files); i++) {
+            string_obj* str = vector_item(files, i);
+            string_delete(str);
         }
+        vector_clear(files);
 
         /// 補完候補をディレクトリから読み込み ///
         if(strcmp(string_c_str(gEditingDir), "") != 0) {
@@ -566,7 +585,9 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                             }
                         }
                         xstrncpy(buf, des, PATH_MAX);
+                        FREE(des);
                         }
+
 #else
                         xstrncpy(buf, entry->d_name, PATH_MAX);
 #endif
@@ -616,6 +637,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                             }
                         }
                         xstrncpy(buf, des, PATH_MAX);
+                        FREE(des);
                         }
 #else
                         xstrncpy(buf, entry->d_name, PATH_MAX);
@@ -664,6 +686,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
             saphire_get_quoted_fname(path, tmp2);
             char* ret = strdup(string_c_str(tmp2));
 
+            FREE(text2);
             return ret;
         }
 
@@ -692,6 +715,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                     rl_completion_append_character= ' ';
                 }
 
+                FREE(text2);
                 return ret; 
             }
         }
@@ -719,6 +743,7 @@ char* readline_file_completion_migemo(const char* text, int rstat)
                 saphire_get_quoted_fname(ret, tmp);
                 free(ret);
 
+                FREE(text2);
                 return strdup(string_c_str(tmp));
             }
         }
@@ -727,9 +752,12 @@ char* readline_file_completion_migemo(const char* text, int rstat)
     if(index == 0 && vector_size(files) == 0) {
         index ++;
         rl_completion_append_character= 0;
+        FREE(text2);
         return strdup("");
     }
 
+    FREE(text2);
+
     return NULL;
 }
 #endif
@@ -1010,25 +1038,30 @@ typedef struct {
     sStatments* mStatments;
 } sUserRegex;
 
-sUserRegex* gUserCompRegex[1024];
-int gUserCompRegexNum = 0;
+vector_obj* gUserCompRegex;
 
 sUserRegex* sUserRegex_new(regex_t* regex, sStatments* statments)
 {
     sUserRegex* self = MALLOC(sizeof(sUserRegex));
 
     self->mRegex = regex;
-    self->mStatments = statments;
+    self->mStatments = STATMENTS_NEW2(statments);
 
     return self;
 }
 
+void sUserRegex_delete(sUserRegex* self)
+{
+    sStatments_delete(self->mStatments);
+    FREE(self);
+}
+
 static vector_obj* gUserCompletions;
 
 static string_obj* gUserCompletionStatment;
 static sStatments* gUserCompletionStatments;
 
-char* readline_user_completion(const char* text, int stat)
+char* readline_completion(const char* text, int stat)
 {
     static int index, wordlen;
 
@@ -1036,10 +1069,17 @@ char* readline_user_completion(const char* text, int stat)
         wordlen = strlen(text);
         index = 0;
 
+        int k;
+        for(k=0; k<vector_size(gUserCompletions); k++) {
+            string_obj* str = vector_item(gUserCompletions, k);
+            string_delete(str);
+        }
         vector_clear(gUserCompletions);
 
-        sWFd* pipeout = sWFd_new(-1);
-        sRFd* pipein = sRFd_new2(-1, string_c_str(gUserCompletionStatment));
+        sWFd* pipeout = WFD_NEW(-1);
+        sRFd* pipein = RFD_NEW2(-1, string_c_str(gUserCompletionStatment));
+        string_delete(gUserCompletionStatment);
+
         int rcode = saphire_run(gUserCompletionStatments, "user completion", pipeout, pipein,  2);
         if(rcode >= 0) {
             char* p = pipeout->mBuffer;
@@ -1062,6 +1102,9 @@ char* readline_user_completion(const char* text, int stat)
                 if(string_c_str(str)[0] != 0) {
                     vector_add(gUserCompletions, str);
                 }
+                else {
+                    string_delete(str);
+                }
 
                 if(*p == 0) break;
             }
@@ -1070,8 +1113,12 @@ char* readline_user_completion(const char* text, int stat)
             fprintf(stderr, "run time err\n");
             fprintf(stderr, "%s", string_c_str(gErrMsg));
 
+            sWFd_delete(pipeout);
+            sRFd_delete(pipein);
             return NULL;
         }
+        sWFd_delete(pipeout);
+        sRFd_delete(pipein);
     }
 
     while(index < vector_size(gUserCompletions)) {
@@ -1147,7 +1194,6 @@ char* readline_ref_completion(const char* text, int stat)
 
 char* readline_file_completion(const char* text, int rstat)
 {
-
     /// クォートを除外 ///
     char* text2 = MALLOC(strlen(text) +1);
     char* _p = (char*)text;
@@ -1176,6 +1222,7 @@ char* readline_file_completion(const char* text, int rstat)
         char* tmp = MALLOC(string_length(gEditingDir) + 1);
         parentname2(tmp, string_length(gEditingDir) + 1, string_c_str(gEditingDir));
         string_put(gEditingDir, tmp);
+        FREE(tmp);
 
         /// チルダから始まっている場合 ///
         if(string_c_str(gEditingDir)[0] == '~') {
@@ -1222,12 +1269,12 @@ char* readline_file_completion(const char* text, int rstat)
         }
 
         /// 補完候補を初期化 ///
-        if(files == NULL) {
-            files = VECTOR_NEW(50);
-        }
-        else {
-            vector_clear(files);
+        int i;
+        for(i=0;i<vector_size(files); i++) {
+            string_obj* str = vector_item(files, i);
+            string_delete(str);
         }
+        vector_clear(files);
 
         /// 補完候補をディレクトリから読み込み ///
         if(strcmp(string_c_str(gEditingDir), "") != 0) {
@@ -1261,6 +1308,7 @@ char* readline_file_completion(const char* text, int rstat)
                             }
                         }
                         xstrncpy(buf, des, PATH_MAX);
+                        FREE(des);
                         }
 #else
                         xstrncpy(buf, entry->d_name, PATH_MAX);
@@ -1309,6 +1357,7 @@ char* readline_file_completion(const char* text, int rstat)
                             }
                         }
                         xstrncpy(buf, des, PATH_MAX);
+                        FREE(des);
                         }
 #else
                         xstrncpy(buf, entry->d_name, PATH_MAX);
@@ -1353,6 +1402,7 @@ char* readline_file_completion(const char* text, int rstat)
                 rl_completion_append_character= ' ';
             }
 
+            FREE(text2);
             return ret;
         }
 
@@ -1365,6 +1415,7 @@ char* readline_file_completion(const char* text, int rstat)
                 string_obj* tmp2 = STRING_NEW("");
                 saphire_get_quoted_fname(path, tmp2);
                 char* ret = strdup(string_c_str(tmp2));
+                string_delete(tmp2);
                 xstrncpy(path, string_c_str(gEditingDir), PATH_MAX);
                 xstrncat(path, file, PATH_MAX);
                 struct stat stat_;
@@ -1380,6 +1431,7 @@ char* readline_file_completion(const char* text, int rstat)
                     rl_completion_append_character= ' ';
                 }
 
+                FREE(text2);
                 return ret; 
             }
         }
@@ -1388,9 +1440,11 @@ char* readline_file_completion(const char* text, int rstat)
     if(index == 0 && vector_size(files) == 0) {
         index ++;
         rl_completion_append_character= 0;
+        FREE(text2);
         return strdup("");
     }
 
+    FREE(text2);
     return NULL;
 }
 
@@ -1607,6 +1661,13 @@ char** readline_on_complete(const char* text, int start, int end)
                 }
 
                 if(object) {
+                    if(gMethodCompletions) {
+                        int i;
+                        for(i=0; i<vector_size(gMethodCompletions); i++) {
+                            string_delete(vector_item(gMethodCompletions, i));
+                        }
+                        vector_delete(gMethodCompletions);
+                    }
                     gMethodCompletions = saphire_methods_names(object);
                     method = TRUE; 
                     program = FALSE;  
@@ -1835,57 +1896,65 @@ char** readline_on_complete(const char* text, int start, int end)
     }
 
     sStatments* statments = NULL;
-    int i;
-    for(i=0; i< 1024; i++) {
-        sUserRegex* ureg = gUserCompRegex[i];
+    if(strcmp(string_c_str(buf), string_c_str(command)) != 0) {
+        int i;
+        for(i=0; i<vector_size(gUserCompRegex); i++) {
+            sUserRegex* ureg = vector_item(gUserCompRegex, i);
 
-        if(ureg) {
-            regex_t* reg = ureg->mRegex;
-            char* str2 = string_c_str(command);
+            if(ureg) {
+                regex_t* reg = ureg->mRegex;
+                char* str2 = string_c_str(command);
 
-            OnigRegion* region = onig_region_new();
-            int r2 = onig_search(reg, str2
-               , str2 + strlen(str2)
-               , str2, str2 + strlen(str2)
-               , region, ONIG_OPTION_NONE);
-            onig_region_free(region, 1);
+                OnigRegion* region = onig_region_new();
+                int r2 = onig_search(reg, str2
+                   , str2 + strlen(str2)
+                   , str2, str2 + strlen(str2)
+                   , region, ONIG_OPTION_NONE);
+                onig_region_free(region, 1);
 
-            if(r2 >= 0) {
-                statments = ureg->mStatments;
-                break;
+                if(r2 >= 0) {
+                    statments = ureg->mStatments;
+                    break;
+                }
             }
         }
     }
+    string_delete(command);
+    string_delete(buf);
 
-/*
     if(statments) {
         gUserCompletionStatment = statment;
-        gUserCompletionStatments = sStatments_new2(statments);
-        return rl_completion_matches(text, readline_user_completion);
+        gUserCompletionStatments = statments;
+        return rl_completion_matches(text, readline_completion);
     }
-*/
-    if(user) {
+    else if(user) {
         rl_completion_append_character= 0;
+        string_delete(statment);
         return rl_completion_matches(text, rl_username_completion_function);
     }
     else if(ref) {
         gDerefCompletionProgram = program;
+        string_delete(statment);
         return rl_completion_matches(text, readline_deref_completion);
     }
     else if(method) {
         rl_completion_append_character= ' ';
+        string_delete(statment);
         return rl_completion_matches(text, readline_method_completion);
     }
     else if(program) {
         rl_completion_append_character= ' ';
+        string_delete(statment);
         return rl_completion_matches(text, readline_program_completion);
     }
 #if defined(HAVE_MIGEMO_H)
     else {
+        string_delete(statment);
         return rl_completion_matches(text, readline_file_completion_migemo);
     }
 #else
     else {
+        string_delete(statment);
         return rl_completion_matches(text, readline_file_completion);
     }
 #endif
@@ -1943,17 +2012,23 @@ static int readline_macro(int count, int key)
     tmp[rl_point] = 0;
 
     string_obj* ret = STRING_NEW("");
-    sRFd* rfd = sRFd_new(-1);
+    sRFd* rfd = RFD_NEW(-1);
     sRFd_set_buffer(rfd, tmp);
+    FREE(tmp);
     if(saphire_shell3(ret, "macro", "SAPHIRE_MACRO", rfd) < 0) {
+        string_delete(ret);
+        sRFd_delete(rfd);
         fprintf(stderr, "run time err\n");
         fprintf(stderr, "%s", string_c_str(gErrMsg));
         fprintf(stderr, "please press cntrol + l....\n");
         return 0;
     }
+    sRFd_delete(rfd);
 
     rl_insert_text(string_c_str(ret));
 
+    string_delete(ret);
+
     return 0;
 }
 
@@ -2059,19 +2134,19 @@ BOOL commands_remove_history(int* rcode, vector_obj* argv, vector_obj* blocks, v
 }
 
 
-BOOL commands_user_completion(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
+BOOL commands_completion(int* rcode, vector_obj* argv, vector_obj* blocks, vector_obj* parent_blocks, sWFd* nextout, sRFd* nextin, int nexterr, char* title, BOOL input, char* sname, int sline)
 {
     if(vector_size(argv) >= 2 && vector_size(blocks) == 1) {
         int i;
         for(i=1; i<vector_size(argv); i++) {
             char* regex = string_c_str(vector_item(argv, i));
             sBlock* block = vector_item(blocks, 0);
-            sStatments* statments = sStatments_new2(block->mStatments);
+            sStatments* statments = block->mStatments;
 
             int r;
             regex_t* reg = get_reg(&r, regex);
             if(r == ONIG_NORMAL) {
-                gUserCompRegex[gUserCompRegexNum++] = sUserRegex_new(reg, statments);
+                vector_add(gUserCompRegex, sUserRegex_new(reg, statments));
                 *rcode = 0;
             }
             else {
@@ -2087,11 +2162,11 @@ BOOL commands_user_completion(int* rcode, vector_obj* argv, vector_obj* blocks,
 static void read_rc_obj_file(BOOL run_user_runtimescript)
 {
     char rc_fname[PATH_MAX];
-    snprintf(rc_fname, PATH_MAX, "%s/user_compl.sao", SYSCONFDIR);
+    snprintf(rc_fname, PATH_MAX, "%s/completion.sao", SYSCONFDIR);
 
     if(access(rc_fname, R_OK) == 0) {
-        sWFd* pipeout = sWFd_new(STDOUT_FILENO);
-        sRFd* pipein = sRFd_new(STDIN_FILENO);
+        sWFd* pipeout = WFD_NEW(STDOUT_FILENO);
+        sRFd* pipein = RFD_NEW(STDIN_FILENO);
         saphire_init_stack_frame();
         if(saphire_load_obj(rc_fname
            , pipeout, pipein, STDERR_FILENO, NULL) < 0)
@@ -2103,6 +2178,8 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
             fprintf(stderr, "can't flash rc");
             exit(1);
         }
+        sWFd_delete(pipeout);
+        sRFd_delete(pipein);
     }
     else {
         fprintf(stderr, "can't find %s file\n", rc_fname);
@@ -2114,8 +2191,8 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
         char* user_rc_fname = getenv("SAPHIRE_COMPL_USERRC");
         if(user_rc_fname) {
             if(access(user_rc_fname, R_OK) == 0) {
-                sWFd* pipeout = sWFd_new(STDOUT_FILENO);
-                sRFd* pipein = sRFd_new(STDIN_FILENO);
+                sWFd* pipeout = WFD_NEW(STDOUT_FILENO);
+                sRFd* pipein = RFD_NEW(STDIN_FILENO);
                 saphire_init_stack_frame();
                 if(saphire_load_obj(user_rc_fname
                     , pipeout, pipein, STDERR_FILENO, NULL) < 0)
@@ -2124,6 +2201,8 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
                     exit(1);
                 }
                 sWFd_flash(pipeout);
+                sRFd_delete(pipein);
+                sWFd_delete(pipeout);
             }
         }
     }
@@ -2131,7 +2210,7 @@ static void read_rc_obj_file(BOOL run_user_runtimescript)
 
 void readline_init()
 {
-    memset(gUserCompRegex, 0, sizeof(gUserCompRegex));
+    gUserCompRegex = VECTOR_NEW(100);
     gUserCompletions = VECTOR_NEW(50);
 
     gUserCompletionStatment = NULL;
@@ -2160,12 +2239,40 @@ void readline_init()
     saphire_add_inner_command("write_history", commands_write_history);
     saphire_add_inner_command("clear_history", commands_clear_history);
     saphire_add_inner_command("remove_history", commands_remove_history);
-    saphire_add_inner_command("user_completion", commands_user_completion);
-    //read_rc_obj_file(TRUE);
+    saphire_add_inner_command("completion", commands_completion);
+    read_rc_obj_file(TRUE);
+
+    files = VECTOR_NEW(30);
 }
 
 void readline_final()
 {
+    if(gMethodCompletions) {
+        int i;
+        for(i=0; i<vector_size(gMethodCompletions); i++) {
+            string_delete(vector_item(gMethodCompletions, i));
+        }
+        vector_delete(gMethodCompletions);
+    }
+    int l;
+    for(l=0; l<vector_size(gUserCompRegex); l++) {
+        sUserRegex_delete(vector_item(gUserCompRegex, l));
+    }
+    vector_delete(gUserCompRegex);
+
+    int k;
+    for(k=0; k<vector_size(gUserCompletions); k++) {
+        string_obj* str = vector_item(gUserCompletions, k);
+        string_delete(str);
+    }
+    vector_delete(gUserCompletions);
+    int i;
+    for(i=0;i<vector_size(files); i++) {
+        string_obj* str = vector_item(files, i);
+        string_delete(str);
+    }
+    vector_delete(files);
+    string_delete(gEditingDir);
 #if defined(HAVE_MIGEMO_H)
     migemo_final();
 #endif
