@@ -49,16 +49,19 @@ typedef struct
 
 sStatments* sStatments_new();
 sStatments* sStatments_new2(sStatments* statments);
+void sStatments_delete(sStatments* self);
 
 typedef struct _sClass {
     string_obj* name;      // クラス名
     sStatments* statments; // 命令表
     struct _sClass* parent;        // オーバーライド先
     BOOL input;             // 実行時の情報
+
+    int mRefCount;
 } sClass;
 
-sClass* sClass_new(char* name, sStatments* statments, sClass* parent);
-sClass* sClass_new2(sClass* klass);
+sClass* sClass_new(char* name, sStatments* statments, sClass* parent, int ref_count);
+void sClass_delete(sClass* self);
   
 void sClass_save(sClass* self, int fd);
 sClass* sClass_load(int fd);
@@ -66,7 +69,7 @@ void sClass_view(sClass* self);
 /// オブジェクトの定義
 typedef struct _sObject {
     string_obj* mName;               /// 自分の名前
-    sClass* mClass;                  /// もしあるなら属しているクラス
+    string_obj* mClassName;          /// もしあるなら属しているクラス名
     hash_obj* mGlobals;
     hash_obj* mHashs;
     hash_obj* mArrays;
@@ -77,9 +80,12 @@ typedef struct _sObject {
     struct _sObject* mParent;
 
     hash_obj* mMethods;
+
+    int mRefCount;
 } sObject;
 
-sObject* sObject_new(char* name, sClass* klass, sObject* parent);
+sObject* sObject_new(char* name, char* klass, sObject* parent, int ref_count);
+void sObject_delete(sObject* self);
 
 sObject* saphire_get_object(char* name);
     // 引数の名前のオブジェクトを返す
@@ -99,6 +105,9 @@ typedef struct {
 } sRef;
 
 sRef* sRef_new(void* mem, enum eMemCheckerKind kind);
+void sRef_delete(sRef* self);
+void sRef_inc_refcount(sRef* self);
+void sRef_dec_refcount(sRef* self);
 
 sRef* saphire_get_ref(char* name);
     // 引数の名前のリファレンスを返す
@@ -113,7 +122,17 @@ typedef struct {
     int mMallocSize;
 } sWFd;
 
+#ifndef MDEBUG
 sWFd* sWFd_new(int fd);
+
+#define WFD_NEW(o) sWFd_new(o)
+
+#else
+sWFd* sWFd_new_debug(int fd, const char* fname, int line, const char* func);
+#define WFD_NEW(o) sWFd_new_debug(o, __FILE__, __LINE__, __FUNCTION__)
+
+#endif
+
 void sWFd_delete(sWFd* self);
 BOOL sWFd_push_back(sWFd* self, char* str);
 BOOL sWFd_flash(sWFd* self);
@@ -133,8 +152,25 @@ typedef struct {
     int mMallocSize;
 } sRFd;
 
+#if defined(MDEBUG)
+
+sRFd* sRFd_new_debug(int fd, const char* fname, int line, const char* func_name);
+#define RFD_NEW(o) sRFd_new_debug(o, __FILE__, __LINE__, __FUNCTION__)
+
+sRFd* sRFd_new2_debug(int fd, char* str, const char* fname, int line, const char* func_name);
+#define RFD_NEW2(o, o2) sRFd_new2_debug(o, o2, __FILE__, __LINE__, __FUNCTION__)
+
+#else
+
 sRFd* sRFd_new(int fd);
+#define RFD_NEW(o) sRFd_new(o)
+
 sRFd* sRFd_new2(int fd, char* str);
+#define RFD_NEW2(o, o2) sRFd_new2(o, o2)
+
+#endif
+
+
 void sRFd_delete(sRFd* self);
 BOOL sRFd_close(sRFd* self);
 int sRFd_read_all(sRFd* self, string_obj* str);
@@ -372,6 +408,8 @@ int statment_tree_internal_commands_read_nextin_oneline_preserve(sRFd* nextin, s
 void saphire_sweep();
     // ごみ掃除
     // sapire_shellやsaphire_loadが呼ばれた後呼び出してください
+int saphire_gc_sweep(sWFd* nextout);
+    // GCのごみ掃除
 
 extern vector_obj* gSaphireVarCompletions;
 // 変数補完候補
