@@ -195,7 +195,7 @@ typedef struct
     sStatments* contents;
 } sWhile;
 
-sWhile* sWhile_new(MANAGED sStatments* joukensiki, MANAGED sStatments* contents);
+sWhile* sWhile_new(sStatments* joukensiki, sStatments* contents);
 sWhile* sWhile_new2(sWhile* while_);
 
 void sWhile_delete(sWhile* self);
@@ -207,8 +207,9 @@ typedef struct
     sStatments* contents;
 } sSubshell;
 
-sSubshell* sSubshell_new(MANAGED sStatments* contents);
+sSubshell* sSubshell_new(sStatments* contents);
 sSubshell* sSubshell_new2(sSubshell* subshell);
+void sSubshell_delete(sSubshell* self);
 
 void sSubshell_delete(sSubshell* self);
 void sSubshell_save(sSubshell* self, int fd);
@@ -221,7 +222,7 @@ typedef struct
     vector_obj* contents_list;
 } sIf;
 
-sIf* sIf_new(MANAGED vector_obj* joukensiki_list, MANAGED vector_obj* contents_list);
+sIf* sIf_new(vector_obj* joukensiki_list, vector_obj* contents_list);
 sIf* sIf_new2(sIf* if_);
 
 void sIf_delete(sIf* self);
@@ -239,9 +240,30 @@ typedef struct
     void* mBody;
 } sArg; // mArgs
 
-sArg* sArg_new(int kind, void* body);
+
+#ifdef MDEBUG
+
+sArg* sArg_new_debug(int kind, MANAGED void* body, const char* fname, int line, const char* func);
+
+#define ARG_NEW(o, o2) sArg_new_debug(o, o2, __FILE__, __LINE__, __FUNCTION__)
+
+sArg* sArg_new2_debug(sArg* arg, const char* fname, int line, const char* func);
+
+#define ARG_NEW2(o) sArg_new2_debug(o, __FILE__, __LINE__, __FUNCTION__)
+
+#else
+
+sArg* sArg_new(int kind, MANAGED void* body);
+
+#define ARG_NEW(o, o2) sArg_new_debug(o, o2)
+
 sArg* sArg_new2(sArg* arg);
-void sArg_delete(sArg* arg);
+
+#define ARG_NEW2(o) sArg_new2_debug(o)
+
+#endif
+
+void sArg_delete(sArg* self);
 
 void sArg_save(sArg* self, int fd);
 sArg* sArg_load(int fd);
@@ -257,6 +279,7 @@ typedef struct {
 
 sRedirect* sRedirect_new(enum eRedirect type, int fd);
 sRedirect* sRedirect_new2(sRedirect* redirect);
+void sRedirect_delete(sRedirect* self);
 void sRedirect_save(sRedirect* self, int fd);
 sRedirect* sRedirect_load(int fd);
 void sRedirect_delete(sRedirect* self);
@@ -274,6 +297,7 @@ typedef struct
 } sPSub;
 
 sPSub* sPSub_new(char* fname, sStatments* statments);
+void sPSub_delete(sPSub* self);
 
 typedef struct
 {
@@ -288,13 +312,22 @@ typedef struct
     void* mExtra;
 } sCommand;
 
-#ifdef MDEBUG 
-void sCommand_view(sCommand* self);
-#endif
+
+#ifndef MDEBUG
 
 sCommand* sCommand_new();
 #define COMMAND_NEW() sCommand_new()
 
+void sCommand_view(sCommand* self);
+
+#else
+
+sCommand* sCommand_new_debug(const char* fname, int line, const char* func);
+
+#define COMMAND_NEW() sCommand_new_debug(__FILE__, __LINE__, __FUNCTION__)
+
+#endif
+
 void sCommand_save(sCommand* self, int fd);
 sCommand* sCommand_load(int fd);
 void sCommand_delete(sCommand* self);
@@ -326,20 +359,32 @@ typedef struct
 } sStatment;
 
 #ifdef MDEBUG
-void sStatment_view(sStatment* self);
-#endif
-
+sStatment* sStatment_new_debug(const char* fname, int line, const char* func_name);
+#define STATMENT_NEW() sStatment_new_debug(__FILE__, __LINE__, __FUNCTION__)
+sStatment* sStatment_new2_debug(sStatment* statment, const char* fname, int line, const char* func_name);
+#define STATMENT_NEW2(o) sStatment_new2_debug(o, __FILE__, __LINE__, __FUNCTION__)
+#else
 sStatment* sStatment_new();
 #define STATMENT_NEW() sStatment_new()
-
-sStatment* sStatment_new2(sStatment* statment);
+sStatment* sStatment_new2();
+#define STATMENT_NEW2(o) sStatment_new2(o)
+#endif
 
 void sStatment_delete(sStatment* self);
 void sStatment_save(sStatment* self, int fd);
 sStatment* sStatment_load(int fd);
 
+#if defined(MDEBUG)
+sStatments* sStatments_new_debug(const char* fname, int line, const char* func_name);
+#define STATMENTS_NEW() sStatments_new_debug(__FILE__, __LINE__, __FUNCTION__)
+sStatments* sStatments_new2_debug(sStatments* statments, const char* fname, int line, const char* func_name);
+#define STATMENTS_NEW2(o) sStatments_new2_debug(o, __FILE__, __LINE__, __FUNCTION__)
+#else
 sStatments* sStatments_new();
 #define STATMENTS_NEW() sStatments_new()
+sStatment* sStatments_new2(sStatments* statments);
+#define STATMENTS_NEW2(o) sStatments_new2(o)
+#endif
 
 void sStatments_view(sStatments* self);
 
@@ -355,10 +400,11 @@ typedef struct _sFunction {
     BOOL input;             // ¹Իξ
 
     struct _sFunction* parent;     // С饤ɤȤο
+
+    int mRefCount;
 } sFunction;
 
-sFunction* sFunction_new(char* name, MANAGED sStatments* statments, char* arg_name, sFunction* parent);
-sFunction* sFunction_new2(sFunction* func);
+sFunction* sFunction_new(char* name, sStatments* statments, char* arg_name, sFunction* parent, int ref_count);
   
 void sFunction_delete(sFunction* self);
 void sFunction_save(sFunction* self, int fd);
@@ -382,10 +428,42 @@ extern hash_obj* gObjects;
 extern hash_obj* gRefs;
 // ֥ ϥ֥̾ ƤsObject*
 
+typedef struct {
+    char* mStr;
+    int mLen;
+
+    int mMallocLen;
+
+    int mRefCount;
+} sGlobal;
+
+typedef struct {
+   hash_it** mTable;
+   int mTableSize;
+
+   hash_it* mEntryIt;
+
+   int mCounter;                // 
+   int mRefCount;
+} sHash;
+
+typedef struct {
+   void** mTable;
+   int mTableSize;
+   
+   int mCount;
+
+   int mRefCount;
+} sAry;
+
 string_obj* sVar_new(char* str);
-string_obj* sGlobal_new(char* str);
-vector_obj* sAry_new(int first_size);
-hash_obj* sHash_new(int size);
+void sVar_delete(string_obj* self);
+string_obj* sGlobal_new(char* str, int ref_count);
+void sGlobal_delete(string_obj* self);
+vector_obj* sAry_new(int first_size, int ref_count);
+void sAry_delete(vector_obj* self);
+hash_obj* sHash_new(int size, int ref_count);
+void sHash_delete(hash_obj* self);
 
 typedef struct {
     sStatments* mStatments;
@@ -394,6 +472,7 @@ typedef struct {
 
 sAtCommand* sAtCommand_new(sStatments* statments, enum eLineField lf);
 sAtCommand* sAtCommand_new2(sAtCommand* at_command);
+void sAtCommand_delete(sAtCommand* self);
 sAtCommand* sAtCommand_save(sAtCommand* self, int fd);
 sAtCommand* sAtCommand_load(int fd);
 
@@ -410,14 +489,6 @@ BOOL expand_env(char* p, string_obj* out, sRFd* rfd, char* sname, int* line, sOb
 BOOL parse(char* cmdline, char* sname, int* line, sStatments* statments);
 BOOL delete_comments(char* p, string_obj* cmdline2, char* separator, char* sname, int* line);
 
-typedef struct {
-    int number;
-    int number2;
-    int number_num;
-} sIndex;
-
-sIndex* sIndex_new(int num, int num2, int num_num);
-void sIndex_delete(sIndex* self);
 
 BOOL read_until_next_cbrace_end(char** p, string_obj* not_evaled, int* read_end_of_statment, BOOL line_inc, char* sname, int* line);
 
